@@ -216,6 +216,15 @@ public class MediaLibrary {
             while (!directories.isEmpty()) {
                 File dir = directories.pop();
                 File[] f = null;
+                /* .nomedia tells media players on Android to skip the
+                 * folder in the media library because they don't contain
+                 * useful music, such as notification sounds,
+                 * navigation voice phrases etc.
+                 */
+                if(new File(dir.getAbsolutePath() + "/.nomedia").exists()) {
+                    continue;
+                }
+
                 if ((f = dir.listFiles(mediaFileFilter)) != null) {
                     for (int i = 0; i < f.length; i++) {
                         File file = f[i];
@@ -255,9 +264,7 @@ public class MediaLibrary {
                 for (String fileURI : addedLocations) {
                     existingMedias.remove(fileURI);
                 }
-                for (String existingMedia : existingMedias.keySet()) {
-                    mDBManager.removeMedia(existingMedia);
-                }
+                mDBManager.removeMedias(existingMedias.keySet());
             }
 
             // hide progressbar in header
@@ -283,7 +290,8 @@ public class MediaLibrary {
                     int dotIndex = fileName.lastIndexOf(".");
                     if (dotIndex != -1) {
                         String fileExt = fileName.substring(dotIndex);
-                        accepted = Media.EXTENTIONS.contains(fileExt);
+                        accepted = Media.AUDIO_EXTENSIONS.contains(fileExt) ||
+                                   Media.VIDEO_EXTENSIONS.contains(fileExt);
                     }
                 }
             }
