@@ -47,7 +47,7 @@ public class DirectoryAdapter extends BaseAdapter {
     public final static String TAG = "VLC/DirectoryAdapter";
 
     public static boolean acceptedPath(String f) {
-        return Pattern.compile(Media.EXTENTIONS_REGEX, Pattern.CASE_INSENSITIVE).matcher(f).matches();
+        return Pattern.compile(Media.EXTENSIONS_REGEX, Pattern.CASE_INSENSITIVE).matcher(f).matches();
     }
 
     /**
@@ -267,13 +267,13 @@ public class DirectoryAdapter extends BaseAdapter {
 
             if(folderCount > 0)
                 holderText += context.getResources().getQuantityString(
-                        R.plurals.subfolders, folderCount, folderCount
+                        R.plurals.subfolders_quantity, folderCount, folderCount
                 );
             if(folderCount > 0 && songCount > 0)
                 holderText += ", ";
             if(songCount > 0)
                 holderText += context.getResources().getQuantityString(
-                        R.plurals.songs, songCount, songCount
+                        R.plurals.songs_quantity, songCount, songCount
                 );
         }
         holder.text.setText(holderText);
@@ -282,6 +282,10 @@ public class DirectoryAdapter extends BaseAdapter {
         else
             holder.icon.setImageResource(R.drawable.ic_folder);
 
+        if(selectedNode.isFile())
+            v.setLongClickable(true);
+        else
+            v.setLongClickable(false);
         return v;
     }
 
@@ -305,12 +309,25 @@ public class DirectoryAdapter extends BaseAdapter {
         return true;
     }
 
+    public boolean isChildFile(int position) {
+        DirectoryAdapter.Node selectedNode = mCurrentNode.children.get(position);
+        return selectedNode.isFile();
+    }
+
     public String getMediaLocation(int position) {
         return LibVLC.getExistingInstance().nativeToURI(
                 this.mCurrentDir + "/" + mCurrentNode.children.get(position).name
         );
     }
 
+    public ArrayList<String> getAllMediaLocations() {
+        ArrayList<String> a = new ArrayList<String>();
+        // i = 1 to exclude ".." folder
+        for(int i = 1; i < mCurrentNode.children.size(); i++)
+            a.add(getMediaLocation(i));
+        return a;
+    }
+
     public void clear() {
         for(DirectoryAdapter.Node n : this.mCurrentNode.children)
             n.children.clear();
