@@ -20,9 +20,6 @@
 
 package org.videolan.vlc.gui.audio;
 
-import java.io.File;
-import java.net.URI;
-import java.net.URISyntaxException;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
@@ -31,12 +28,14 @@ import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.Media;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
+import org.videolan.vlc.VLCCallbackTask;
+import org.videolan.vlc.WeakHandler;
+import org.videolan.vlc.gui.CommonDialogs;
 import org.videolan.vlc.interfaces.ISortable;
 import org.videolan.vlc.widget.FlingViewGroup;
 import org.videolan.vlc.widget.FlingViewGroup.ViewSwitchListener;
 
 import android.app.AlertDialog;
-import android.content.DialogInterface;
 import android.content.Intent;
 import android.graphics.Color;
 import android.os.Bundle;
@@ -98,12 +97,11 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         mAudioController = AudioServiceController.getInstance();
 
         mMediaLibrary = MediaLibrary.getInstance(getActivity());
-        mMediaLibrary.addUpdateHandler(mHandler);
 
         mSongsAdapter = new AudioListAdapter(getActivity());
-        mArtistsAdapter = new AudioPlaylistAdapter(getActivity(), R.plurals.albums, R.plurals.songs);
-        mAlbumsAdapter = new AudioPlaylistAdapter(getActivity(), R.plurals.songs, R.plurals.songs);
-        mGenresAdapter = new AudioPlaylistAdapter(getActivity(), R.plurals.albums, R.plurals.songs);
+        mArtistsAdapter = new AudioPlaylistAdapter(getActivity(), R.plurals.albums_quantity, R.plurals.songs_quantity);
+        mAlbumsAdapter = new AudioPlaylistAdapter(getActivity(), R.plurals.songs_quantity, R.plurals.songs_quantity);
+        mGenresAdapter = new AudioPlaylistAdapter(getActivity(), R.plurals.albums_quantity, R.plurals.songs_quantity);
     }
 
     @Override
@@ -156,6 +154,18 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         updateLists();
     }
 
+    @Override
+    public void onPause() {
+        super.onPause();
+        mMediaLibrary.removeUpdateHandler(mHandler);
+    }
+
+    @Override
+    public void onResume() {
+        super.onResume();
+        mMediaLibrary.addUpdateHandler(mHandler);
+    }
+
     OnItemClickListener songListener = new OnItemClickListener() {
         @Override
         public void onItemClick(AdapterView<?> av, View v, int p, long id) {
@@ -222,32 +232,6 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         }
     };
 
-    public void deleteMedia( final List<String> addressMedia, final Media aMedia ) {
-        AlertDialog alertDialog = new AlertDialog.Builder(getActivity())
-        .setTitle(R.string.confirm_delete)
-        .setMessage(R.string.validation)
-        .setIcon(android.R.drawable.ic_dialog_alert)
-        .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
-            @Override
-            public void onClick(DialogInterface dialog, int whichButton) {
-                URI adressMediaUri = null;
-                try {
-                    adressMediaUri = new URI (addressMedia.get(0));
-                } catch (URISyntaxException e) {
-                    // TODO Auto-generated catch block
-                    e.printStackTrace();
-                }
-                File fileMedia =  new File(adressMediaUri);
-                fileMedia.delete();
-                mMediaLibrary.getMediaItems().remove(aMedia);
-                updateLists();
-            }
-        })
-        .setNegativeButton(android.R.string.cancel, null).create();
-
-        alertDialog.show();
-    }
-
     @Override
     public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
         MenuInflater inflater = getActivity().getMenuInflater();
@@ -285,7 +269,20 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
         }
 
         if (id == R.id.audio_list_browser_delete) {
-            deleteMedia(mSongsAdapter.getLocation(groupPosition), mSongsAdapter.getItem(groupPosition));
+            AlertDialog alertDialog = CommonDialogs.deleteMedia(getActivity(), mSongsAdapter.getLocation(groupPosition).get(0),
+                    new VLCCallbackTask(new VLCCallbackTask.CallbackListener() {
+
+                @Override
+                public void callback_object(Object o) {
+                    Media aMedia = (Media)o;
+                    mMediaLibrary.getMediaItems().remove(aMedia);
+                    updateLists();
+                }
+
+                @Override
+                public void callback() { }
+            }, mSongsAdapter.getItem(groupPosition)));
+            alertDialog.show();
             return true;
         }
 
@@ -327,7 +324,6 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
 
     @Override
     public void onDestroy() {
-        mMediaLibrary.removeUpdateHandler(mHandler);
         mSongsAdapter.clear();
         mArtistsAdapter.clear();
         mAlbumsAdapter.clear();
@@ -366,16 +362,22 @@ public class AudioBrowserFragment extends SherlockFragment implements ISortable
     /**
      * Handle changes on the list
      */
-    protected Handler mHandler = new Handler() {
+    private Handler mHandler = new AudioBrowserHandler(this);
+
+    private static class AudioBrowserHandler extends WeakHandler<AudioBrowserFragment> {
+        public AudioBrowserHandler(AudioBrowserFragment owner) {
+            super(owner);
+        }
 
         @Override
         public void handleMessage(Message msg) {
+            AudioBrowserFragment fragment = getOwner();
             switch (msg.what) {
                 case MediaLibrary.MEDIA_ITEMS_UPDATED:
-                updateLists();
-                break;
+                    fragment.updateLists();
+                    break;
+            }
         }
-    }
     };
 
     private final Comparator<Media> byMRL = new Comparator<Media>() {
