@@ -31,6 +31,7 @@ import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.Media;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
+import org.videolan.vlc.WeakHandler;
 
 import android.app.AlertDialog;
 import android.content.DialogInterface;
@@ -76,7 +77,6 @@ public class AudioListFragment extends SherlockListFragment {
         mAudioController = AudioServiceController.getInstance();
 
         mMediaLibrary = MediaLibrary.getInstance(getActivity());
-        mMediaLibrary.addUpdateHandler(mHandler);
 
         mSongsAdapter = new AudioListAdapter(getActivity());
         setListAdapter(mSongsAdapter);
@@ -96,8 +96,19 @@ public class AudioListFragment extends SherlockListFragment {
     }
 
     @Override
-    public void onDestroy() {
+    public void onPause() {
+        super.onPause();
         mMediaLibrary.removeUpdateHandler(mHandler);
+    }
+
+    @Override
+    public void onResume() {
+        super.onResume();
+        mMediaLibrary.addUpdateHandler(mHandler);
+    }
+
+    @Override
+    public void onDestroy() {
         mSongsAdapter.clear();
         super.onDestroy();
     }
@@ -188,13 +199,19 @@ public class AudioListFragment extends SherlockListFragment {
     /**
      * Handle changes on the list
      */
-    protected Handler mHandler = new Handler() {
+    private Handler mHandler = new AudioListHandler(this);
+
+    private static class AudioListHandler extends WeakHandler<AudioListFragment> {
+        public AudioListHandler(AudioListFragment owner) {
+            super(owner);
+        }
 
         @Override
         public void handleMessage(Message msg) {
+            AudioListFragment fragment = getOwner();
             switch (msg.what) {
                 case MediaLibrary.MEDIA_ITEMS_UPDATED:
-                    updateList();
+                    fragment.updateList();
                     break;
             }
         }
