@@ -25,12 +25,15 @@ import java.io.UnsupportedEncodingException;
 import java.lang.reflect.Method;
 import java.net.URLDecoder;
 
+import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.EventManager;
 import org.videolan.vlc.LibVLC;
 import org.videolan.vlc.LibVlcException;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
+import org.videolan.vlc.WeakHandler;
 import org.videolan.vlc.gui.PreferencesActivity;
+import org.videolan.vlc.gui.audio.AudioPlayerActivity;
 import org.videolan.vlc.interfaces.IPlayerControl;
 import org.videolan.vlc.interfaces.OnPlayerControlListener;
 import org.videolan.vlc.widget.PlayerControlClassic;
@@ -65,6 +68,7 @@ import android.view.SurfaceHolder.Callback;
 import android.view.SurfaceView;
 import android.view.View;
 import android.view.View.OnClickListener;
+import android.view.View.OnSystemUiVisibilityChangeListener;
 import android.view.ViewGroup.LayoutParams;
 import android.view.WindowManager;
 import android.view.animation.AnimationUtils;
@@ -81,6 +85,7 @@ public class VideoPlayerActivity extends Activity {
     private SurfaceView mSurface;
     private SurfaceHolder mSurfaceHolder;
     private LibVLC mLibVLC;
+    private String mLocation;
 
     private static final int SURFACE_BEST_FIT = 0;
     private static final int SURFACE_FIT_HORIZONTAL = 1;
@@ -99,8 +104,10 @@ public class VideoPlayerActivity extends Activity {
     private static final int SHOW_PROGRESS = 2;
     private static final int SURFACE_SIZE = 3;
     private static final int FADE_OUT_INFO = 4;
+    private static final int HIDE_NAV = 5;
     private boolean mDragging;
     private boolean mShowing;
+    private int mUiVisibility = -1;
     private SeekBar mSeekbar;
     private TextView mTitle;
     private TextView mSysTime;
@@ -114,6 +121,12 @@ public class VideoPlayerActivity extends Activity {
     private ImageButton mLock;
     private ImageButton mSize;
 
+    /**
+     * For uninterrupted switching between audio and video mode
+     */
+    private boolean mSwitchingView;
+    private boolean mEndReached;
+
     // size of the video
     private int mVideoHeight;
     private int mVideoWidth;
@@ -128,11 +141,27 @@ public class VideoPlayerActivity extends Activity {
     private String[] mSubtitleTracks;
 
     @Override
+    @TargetApi(11)
     protected void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
         setContentView(R.layout.player);
 
         SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
+        if(Util.isICSOrLater())
+            getWindow().getDecorView().findViewById(android.R.id.content).setOnSystemUiVisibilityChangeListener(
+                    new OnSystemUiVisibilityChangeListener() {
+                        public void onSystemUiVisibilityChange(int visibility) {
+                            if (visibility == mUiVisibility)
+                                return;
+                            setSurfaceSize(mVideoWidth, mVideoHeight);
+                            if (visibility == View.SYSTEM_UI_FLAG_VISIBLE && !mShowing) {
+                                showOverlay();
+                                mHandler.sendMessageDelayed(mHandler.obtainMessage(HIDE_NAV), OVERLAY_TIMEOUT);
+                            }
+                            mUiVisibility = visibility;
+                        }
+                    }
+            );
 
         /** initialize Views an their Events */
         mOverlayHeader = findViewById(R.id.player_overlay_header);
@@ -178,6 +207,9 @@ public class VideoPlayerActivity extends Activity {
         mAudioManager = (AudioManager) getSystemService(AUDIO_SERVICE);
         mAudioMax = mAudioManager.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
 
+        mSwitchingView = false;
+        mEndReached = false;
+
         registerReceiver(mBatteryReceiver, new IntentFilter(Intent.ACTION_BATTERY_CHANGED));
 
         try {
@@ -200,10 +232,15 @@ public class VideoPlayerActivity extends Activity {
     protected void onStart() {
         super.onStart();
         dimStatusBar(true);
+        mSwitchingView = false;
     }
 
     @Override
     protected void onPause() {
+        if(mSwitchingView) {
+            super.onPause();
+            return;
+        }
         long time = 0;
         if (mLibVLC.isPlaying()) {
             time = mLibVLC.getTime() - 5000;
@@ -212,17 +249,20 @@ public class VideoPlayerActivity extends Activity {
         mSurface.setKeepScreenOn(false);
 
         // Save position
-        SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
-        SharedPreferences.Editor editor = preferences.edit();
-        editor.putLong(PreferencesActivity.LAST_TIME, time);
-        editor.commit();
+        if (time >= 0) {
+            SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
+            SharedPreferences.Editor editor = preferences.edit();
+            editor.putString(PreferencesActivity.LAST_MEDIA, mLocation);
+            editor.putLong(PreferencesActivity.LAST_TIME, time);
+            editor.commit();
+        }
         super.onPause();
     }
 
     @Override
     protected void onDestroy() {
         unregisterReceiver(mBatteryReceiver);
-        if (mLibVLC != null) {
+        if (mLibVLC != null && !mSwitchingView) {
             mLibVLC.stop();
         }
 
@@ -231,9 +271,23 @@ public class VideoPlayerActivity extends Activity {
 
         mAudioManager = null;
 
+        if(mSwitchingView) {
+            Log.d(TAG, "mLocation = \"" + mLocation + "\"");
+            AudioServiceController.getInstance().showWithoutParse(mLocation);
+            Intent i = new Intent(this, AudioPlayerActivity.class);
+            i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_MULTIPLE_TASK);
+            startActivity(i);
+        }
+        //AudioServiceController.getInstance().unbindAudioService(this);
         super.onDestroy();
     }
 
+    @Override
+    protected void onResume() {
+        AudioServiceController.getInstance().bindAudioService(this);
+        super.onResume();
+    }
+
     private final BroadcastReceiver mBatteryReceiver = new BroadcastReceiver()
     {
         @Override
@@ -351,12 +405,26 @@ public class VideoPlayerActivity extends Activity {
         hideInfo(0);
     }
 
+    private void fadeOutInfo() {
+        if (mInfo.getVisibility() == View.VISIBLE)
+            mInfo.startAnimation(AnimationUtils.loadAnimation(
+                    VideoPlayerActivity.this, android.R.anim.fade_out));
+        mInfo.setVisibility(View.INVISIBLE);
+    }
+
     /**
      *  Handle libvlc asynchronous events
      */
-    private final Handler eventHandler = new Handler() {
+    private final Handler eventHandler = new VideoPlayerEventHandler(this);
+
+    private static class VideoPlayerEventHandler extends WeakHandler<VideoPlayerActivity> {
+        public VideoPlayerEventHandler(VideoPlayerActivity owner) {
+            super(owner);
+        }
+
         @Override
         public void handleMessage(Message msg) {
+            VideoPlayerActivity activity = getOwner();
             switch (msg.getData().getInt("event")) {
                 case EventManager.MediaPlayerPlaying:
                     Log.i(TAG, "MediaPlayerPlaying");
@@ -369,51 +437,89 @@ public class VideoPlayerActivity extends Activity {
                     break;
                 case EventManager.MediaPlayerEndReached:
                     Log.i(TAG, "MediaPlayerEndReached");
-                    /* Exit player when reach the end */
-                    VideoPlayerActivity.this.finish();
+                    activity.endReached();
+                    break;
+                case EventManager.MediaPlayerVout:
+                    activity.handleVout(msg);
                     break;
                 default:
                     Log.e(TAG, "Event not handled");
                     break;
             }
-            updateOverlayPausePlay();
+            activity.updateOverlayPausePlay();
         }
     };
 
     /**
      * Handle resize of the surface and the overlay
      */
-    private final Handler mHandler = new Handler() {
+    private final Handler mHandler = new VideoPlayerHandler(this);
+
+    private static class VideoPlayerHandler extends WeakHandler<VideoPlayerActivity> {
+        public VideoPlayerHandler(VideoPlayerActivity owner) {
+            super(owner);
+        }
 
         @Override
         public void handleMessage(Message msg) {
+            VideoPlayerActivity activity = getOwner();
             switch (msg.what) {
                 case FADE_OUT:
-                    hideOverlay(false);
+                    activity.hideOverlay(false);
                     break;
                 case SHOW_PROGRESS:
-                    int pos = setOverlayProgress();
-                    if (!mDragging && mShowing && mLibVLC.isPlaying()) {
+                    int pos = activity.setOverlayProgress();
+                    if (activity.canShowProgress()) {
                         msg = obtainMessage(SHOW_PROGRESS);
                         sendMessageDelayed(msg, 1000 - (pos % 1000));
                     }
                     break;
                 case SURFACE_SIZE:
-                    changeSurfaceSize();
+                    activity.changeSurfaceSize();
                     break;
                 case FADE_OUT_INFO:
-                    if (mInfo.getVisibility() == View.VISIBLE)
-                        mInfo.startAnimation(AnimationUtils.loadAnimation(
-                                VideoPlayerActivity.this, android.R.anim.fade_out));
-                    mInfo.setVisibility(View.INVISIBLE);
+                    activity.fadeOutInfo();
+                    break;
+                case HIDE_NAV:
+                    activity.dimStatusBar(true);
+                    break;
             }
         }
     };
 
+    private boolean canShowProgress() {
+        return !mDragging && mShowing && mLibVLC.isPlaying();
+    }
+
+    private void endReached() {
+        /* Exit player when reach the end */
+        mEndReached = true;
+        finish();
+    }
+
+    private void handleVout(Message msg) {
+        if (msg.getData().getInt("data") == 0 && !mEndReached) {
+            /* Video track lost, open in audio mode */
+            Log.i(TAG, "Video track lost, switching to audio");
+            mSwitchingView = true;
+            finish();
+        }
+    }
+
     private void changeSurfaceSize() {
         // get screen size
-        int dw = getWindowManager().getDefaultDisplay().getWidth();
-        int dh = getWindowManager().getDefaultDisplay().getHeight();
+        int dw = getWindow().getDecorView().getWidth();
+        int dh = getWindow().getDecorView().getHeight();
+
+        // getWindow().getDecorView() doesn't always take orientation into account, we have to correct the values
+        boolean isPortrait = getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT;
+        if (dw > dh && isPortrait || dw < dh && !isPortrait) {
+            int d = dw;
+            dw = dh;
+            dh = d;
+        }
+        if (dw * dh == 0)
+            return;
 
         // calculate aspect ratio
         double ar = (double) mVideoWidth / (double) mVideoHeight;
@@ -500,6 +606,8 @@ public class VideoPlayerActivity extends Activity {
                         showOverlay();
                     } else {
                         hideOverlay(true);
+                        if(Util.isICSOrLater())
+                            mHandler.sendMessageDelayed(mHandler.obtainMessage(HIDE_NAV), OVERLAY_TIMEOUT);
                     }
                 }
                 break;
@@ -718,6 +826,7 @@ public class VideoPlayerActivity extends Activity {
      * show overlay
      */
     private void showOverlay(int timeout) {
+        mHandler.removeMessages(HIDE_NAV);
         mHandler.sendEmptyMessage(SHOW_PROGRESS);
         if (!mShowing) {
             mShowing = true;
@@ -766,14 +875,16 @@ public class VideoPlayerActivity extends Activity {
     }
 
     /**
-     * Dim the status bar and/or navigation icons when needed.
-     * Android 3.0 and later
+     * Dim the status bar and/or navigation icons when needed on Android 3.x.
+     * Hide it on Android 4.0 and later
      */
     @TargetApi(11)
-	private void dimStatusBar(boolean dim) {
-        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.HONEYCOMB) {
+    private void dimStatusBar(boolean dim) {
+        if (Util.isHoneycombOrLater()) {
             if (dim) {
-                mSurface.setSystemUiVisibility(View.SYSTEM_UI_FLAG_LOW_PROFILE);
+                mSurface.setSystemUiVisibility(Util.isICSOrLater()
+                        ? View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
+                        : View.SYSTEM_UI_FLAG_LOW_PROFILE);
             } else {
                 mSurface.setSystemUiVisibility(View.SYSTEM_UI_FLAG_VISIBLE);
             }
@@ -827,36 +938,35 @@ public class VideoPlayerActivity extends Activity {
      *
      */
     private void load() {
-        String location = null;
+        mLocation = null;
         String title = null;
         String lastLocation = null;
         long lastTime = 0;
+        boolean dontParse = false;
         SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
 
         if (getIntent().getAction() != null
                 && getIntent().getAction().equals(Intent.ACTION_VIEW)) {
             /* Started from external application */
-            location = getIntent().getDataString();
-        } else {
+            mLocation = getIntent().getDataString();
+        } else if(getIntent().getExtras() != null) {
             /* Started from VideoListActivity */
-            location = getIntent().getExtras().getString("itemLocation");
+            mLocation = getIntent().getExtras().getString("itemLocation");
+            dontParse = getIntent().getExtras().getBoolean("dontParse");
         }
 
-        if (location != null && location.length() > 0) {
-            mLibVLC.readMedia(location, false);
+        if (mLocation != null && mLocation.length() > 0 && !dontParse) {
+            mLibVLC.readMedia(mLocation, false);
             mSurface.setKeepScreenOn(true);
 
             // Save media for next time, and restore position if it's the same one as before
             lastLocation = preferences.getString(PreferencesActivity.LAST_MEDIA, null);
             lastTime = preferences.getLong(PreferencesActivity.LAST_TIME, 0);
-            SharedPreferences.Editor editor = preferences.edit();
-            editor.putString(PreferencesActivity.LAST_MEDIA, location);
-            editor.commit();
-            if (lastTime > 0 && location.equals(lastLocation))
+            if (lastTime > 0 && mLocation.equals(lastLocation))
                 mLibVLC.setTime(lastTime);
 
             try {
-                title = URLDecoder.decode(location, "UTF-8");
+                title = URLDecoder.decode(mLocation, "UTF-8");
             } catch (UnsupportedEncodingException e) {
             }
             if (title.startsWith("file:")) {
